/*
 * @eventMapToLAS.java Version 1.1 04/03/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package event;

import event.eventFileDataStruct;
import las.lasFileDataStruct;
import iqstrat.iqstratStruct;

/** Class eventMapToLAS
 *  <p> This Class will map the Zonation Log data to the LAS File Well Data.
 *
 *  @version 1.1 04/03/2015
 *  @author  John R. Victorine
 */

public class eventMapToLAS
{
  private static final int _GR   = event.eventStandardTools._GR;
  private static final int _CGR  = event.eventStandardTools._CGR;

  private static final int _PE   = event.eventStandardTools._PE;
  private static final int _RHOB = event.eventStandardTools._RHOB;
  private static final int _DPHI = event.eventStandardTools._DPHI;
  private static final int _NPHI = event.eventStandardTools._NPHI;

  private static final int _SPHI = event.eventStandardTools._SPHI;
  private static final int _DTC  = event.eventStandardTools._DTC;
  private static final int _DTS  = event.eventStandardTools._DTS;
  private static final int _DTSF = event.eventStandardTools._DTSF;
  private static final int _DTSS = event.eventStandardTools._DTSS;

  private static final int _TH   = event.eventStandardTools._TH;
  private static final int _U    = event.eventStandardTools._U;
  private static final int _K    = event.eventStandardTools._K;

  /** Method mapZekeToLAS()
   *  <p> This method will map the Zonation Data Structure to the LAS File
   *      Data Structure.
   *  @param  stIN  = the Zonation File Data Structure
   *  @return stOUT = the LAS File Data Structure
   */

  public static lasFileDataStruct mapZekeToLAS( eventFileDataStruct stIN )
  {
	lasFileDataStruct stOUT = null;

    if (stIN != null)
    {
      if (stIN.iRows > 0)
      {
        stOUT = mergeLogData( stIN );
      }
    }

    return (stOUT);
  }

  /** Method mergeLogData()
   * <p> This method will merge a LAS File Data Structure with the new LAS
   *     File Data Structure
   * @param  st    = Zonation File Data Structure
   * @return stLAS = LAS File Data Structure
   */

  public static lasFileDataStruct mergeLogData( eventFileDataStruct st )
  {
	lasFileDataStruct stLAS = null;

    double depth  = 0.0;
    double depth1 = 0.0;
    double data1  = 0.0;
    double depth2 = 0.0;
    double data2  = 0.0;
    double dTemp  = 0.0;
    int    iTemp  = 0;
    double dStep  = 0.5;
    int    iRows  = 0;

    if (st != null)
    {
      iRows = (int) ( Math.abs(st.dEnd - st.dStart) / dStep ) + 1;

      stLAS            = new lasFileDataStruct();

      stLAS.depthStart = st.dStart;
      stLAS.depthEnd   = st.dEnd;
      stLAS.depthStep  = dStep;
      stLAS.dNull      = st.dNull;

      stLAS.iRows      = iRows;
      stLAS            = las3.las3LoadLogData.initData( iRows, stLAS );

      for (int k=0; k<las.lasStandardTools.TOTAL; k++)
      {
        switch (k)
        {
          case las.lasStandardTools._GR:
            if (st.checkData(_GR)) stLAS.iGR = st.iGR;
            break;
          case las.lasStandardTools._CGR:
            if (st.checkData(_CGR)) stLAS.iCGR = st.iCGR;
            break;

          case las.lasStandardTools._PE:
            if (st.checkData(_PE)) stLAS.iPE = st.iPE;
            break;
          case las.lasStandardTools._RHOB:
            if (st.checkData(_RHOB)) stLAS.iRHOB = st.iRHOB;
            break;
          case las.lasStandardTools._DPHI:
            if (st.checkData(_DPHI)) stLAS.iDPHI = st.iDPHI;
            break;
          case las.lasStandardTools._NPHI:
            if (st.checkData(_NPHI)) stLAS.iNPHI = st.iNPHI;
            break;

          case las.lasStandardTools._DT:
            if (st.checkData(_DTC)) stLAS.iDT = st.iDTC;
            break;
          case las.lasStandardTools._DTS:
            if (st.checkData(_DTS)) stLAS.iDTS = st.iDTS;
            break;
          case las.lasStandardTools._DTSF:
            if (st.checkData(_DTSF)) stLAS.iDTSF = st.iDTSF;
            break;
          case las.lasStandardTools._DTSS:
            if (st.checkData(_DTSS)) stLAS.iDTSS = st.iDTSS;
            break;
          case las.lasStandardTools._SPHI:
            if (st.checkData(_SPHI)) stLAS.iSPHI = st.iSPHI;
            break;

          case las.lasStandardTools._TH:
            if (st.checkData(_TH)) stLAS.iTh = st.iTh;
            break;
          case las.lasStandardTools._U:
            if (st.checkData(_U)) stLAS.iU = st.iU;
            break;
          case las.lasStandardTools._K:
            if (st.checkData(_K)) stLAS.iK = st.iK;
            break;
        }
      }

      for (int j=0; j<stLAS.iRows; j++)
      {
        depth = stLAS.depths[j];

        for (int i=0; i<st.iRows; i++)
        {
		  depth1 = st.dTop[i];
		  if (i == st.iRows-1)
		    depth2 = st.dBase[i];
		  else
		    depth2 = st.dTop[i+1];

		  if ((depth >= depth1) && (depth < depth2))
		  {
            for (int k=0; k<las.lasStandardTools.TOTAL; k++)
            {
              switch (k)
              {
                case las.lasStandardTools._GR:
                  if (st.checkData(_GR))
                    stLAS.setRow(k, j, st.getRow( _GR, i ));
                  break;
                case las.lasStandardTools._CGR:
                  if (st.checkData(_CGR))
                    stLAS.setRow(k, j, st.getRow( _CGR, i ));
                  break;

                case las.lasStandardTools._PE:
                  if (st.checkData(_PE))
                    stLAS.setRow(k, j, st.getRow( _PE, i ));
                  break;
                case las.lasStandardTools._RHOB:
                  if (st.checkData(_RHOB))
                    stLAS.setRow(k, j, st.getRow( _RHOB, i ));
                  break;
                case las.lasStandardTools._DPHI:
                  if (st.checkData(_DPHI))
                    stLAS.setRow(k, j, st.getRow( _DPHI, i ));
                  break;
                case las.lasStandardTools._NPHI:
                  if (st.checkData(_NPHI))
                    stLAS.setRow(k, j, st.getRow( _NPHI, i ));
                  break;

                case las.lasStandardTools._DT:
                  if (st.checkData(_DTC))
                    stLAS.setRow(k, j, st.getRow( _DTC, i ));
                  break;
                case las.lasStandardTools._DTS:
                  if (st.checkData(_DTS))
                    stLAS.setRow(k, j, st.getRow( _DTS, i ));
                  break;
                case las.lasStandardTools._DTSF:
                  if (st.checkData(_DTSF))
                    stLAS.setRow(k, j, st.getRow( _DTSF, i ));
                  break;
                case las.lasStandardTools._DTSS:
                  if (st.checkData(_DTSS))
                    stLAS.setRow(k, j, st.getRow( _DTSS, i ));
                  break;
                case las.lasStandardTools._SPHI:
                  if (st.checkData(_SPHI))
                    stLAS.setRow(k, j, st.getRow( _SPHI, i ));
                  break;

                case las.lasStandardTools._TH:
                  if (st.checkData(_TH))
                    stLAS.setRow(k, j, st.getRow( _TH, i ));
                  break;
                case las.lasStandardTools._U:
                  if (st.checkData(_U))
                    stLAS.setRow(k, j, st.getRow( _U, i ));
                  break;
                case las.lasStandardTools._K:
                  if (st.checkData(_K))
                    stLAS.setRow(k, j, st.getRow( _K, i ));
                  break;
			  }
            }
		  }
		}
	  }
    }

    return (stLAS);
  }

  /** Method mapLASToZeke()
   *  <p> This method will map the Zonation Data Structure with the LAS File
   *      Data Structure to get average data by zones and return the Zonation
   *      Data Structure.
   *  @param  stIN     = the LAS File Data Structure
   *  @param  st       = the Zonation File Data Structure
   *  @param  stStruct = IQSTRAT Data Structure
   *  @return stOUT    = the LAS File Data Structure
   */

  public static eventFileDataStruct mapLASToZeke( lasFileDataStruct stIN,
                                                  eventFileDataStruct st,
                                                  iqstratStruct stStruct )
  {
	eventFileDataStruct stOUT = null;

    if ((stIN != null) && (st != null))
    {
      if (stIN.iRows > 0)
      {
        stOUT = mergeZekeData( stIN, st );
        stOUT = event.eventFileDataUtility.computeMath(stOUT, math.mathLAS._FRESH);
        stOUT = event.eventFileDataUtility.computeSeismicMath(stOUT);
        stOUT = event.eventFileDataUtility.computeLithology(stOUT,
 	                                                        stStruct.stSymbols,
                                                            math.mathLAS._FRESH);
      }
    }

    return (stOUT);
  }

  /** Method mergeZekeData()
   *  <p> This method will merge the data in the Zonation Data Structure with
   *      the data in the LAS File Data Structure to get average data by zones
   *      and return the Zonation Data Structure.
   *  @param stIN  = the LAS File Data Structure
   *  @param st    = the Zonation File Data Structure
   *  @param stOUT = the LAS File Data Structure
   */

  public static eventFileDataStruct mergeZekeData( lasFileDataStruct stIN,
                                                   eventFileDataStruct st )
  {
	int i,j,k,m;
    int    iColumn[] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
    double dSum[]    = { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                         0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 };
    int    iCount[] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
//    double iCount    = 0;
    int    id        = -1;
    double depth1    = 0.0;
    double depth2    = 0.0;
    double dTemp     = 0.0;

    if ((stIN != null) && (st != null))
    {
      for (j=0; j<event.eventStandardTools.TOTAL_LAS; j++)
      {
        if (st.checkData(j))
        {
	      iColumn[j] = 1;
		}
	  }

	  for (i=0; i<st.iRows; i++)
	  {
  	    depth1 = st.dTop[i];
		if (i == st.iRows-1)
		  depth2 = st.dBase[i];
		else
		  depth2 = st.dTop[i+1];

        for (j=0; j<event.eventStandardTools.TOTAL_LAS; j++)
        {
		  dSum[j]   = 0.0;
		  iCount[j] = 0;
		}
//		iCount = 0;

		for (k=0; k<stIN.iRows; k++)
		{
		  if ((stIN.depths[k] >= depth1) && (stIN.depths[k] < depth2))
		  {
//	        iCount++;

            for (m=0; m<las.lasStandardTools.TOTAL; m++)
            {
		      if (isZeke( m, stIN, st))
		      {
			    id = getZekeID( m, stIN, st);
			    if (id > -1)
			    {
				  dTemp = stIN.getRow( m, k );
				  if (dTemp == stIN.dNull)
				    dTemp = 0.0;
				  else
				    iCount[id]++;

			  	  dSum[id] = dSum[id] + dTemp;
//			  	  dSum[id] = dSum[id] + stIN.getRow( m, k );
			    }
			  }
		    }

            for (m=0; m<las.lasStandardTools.TOTAL; m++)
            {
		      if (isZeke( m, stIN, st))
		      {
			    id = getZekeID( m, stIN, st);
			    if (id > -1)
			    {
//				  dTemp = (double) ( (int) (1000.0 * dSum[id]/iCount) ) / 1000.0;
				  dTemp = (double) ( (int) (1000.0 * dSum[id]/iCount[id]) ) / 1000.0;
			  	  st.setRow( id, i, dTemp );
//			  	  st.setRow( id, i, dSum[id]/iCount );
				}
			  }
			}
		  }
		}
	  }

      st = setZekeIdentifiers( stIN, st );
	}

    return (st);
  }

  /** Method isZeke()
   *  <p> This method will turn on the curve identifiers in the Zonation
   *      data structure.
   *  @param  iLAS  = the LAS Column Identifier
   *  @param  stIN  = the LAS File Data Structure
   *  @param  st    = the Zonation File Data Structure
   *  @return bVal  = true the LAS Curve exists, false the LAS Curve does not.
   */

  private static boolean isZeke( int iLAS,
	                             lasFileDataStruct stIN,
                                 eventFileDataStruct st )
  {
	boolean bVal = false;

    if ((stIN != null) && (st != null))
    {
      switch (iLAS)
      {
        case las.lasStandardTools._GR:
          if ((!st.checkData(_GR)) && (stIN.checkData(iLAS)))
          {
	        bVal = true;
          };
          break;
        case las.lasStandardTools._CGR:
          if ((!st.checkData(_CGR)) && (stIN.checkData(iLAS)))
          {
	  	    bVal = true;
          };
          break;

        case las.lasStandardTools._PE:
          if ((!st.checkData(_PE)) && (stIN.checkData(iLAS)))
          {
		    bVal = true;
          };
          break;
        case las.lasStandardTools._RHOB:
          if ((!st.checkData(_RHOB)) && (stIN.checkData(iLAS)))
          {
		    bVal = true;
          };
          break;
        case las.lasStandardTools._DPHI:
          if ((!st.checkData(_DPHI)) && (stIN.checkData(iLAS)))
          {
		    bVal = true;
          };
          break;
        case las.lasStandardTools._NPHI:
          if ((!st.checkData(_NPHI)) && (stIN.checkData(iLAS)))
          {
		    bVal = true;
          };
          break;

        case las.lasStandardTools._DT:
          if ((!st.checkData(_DTC)) && (stIN.checkData(iLAS)))
          {
		    bVal = true;
          };
          break;
        case las.lasStandardTools._DTS:
          if ((!st.checkData(_DTS)) && (stIN.checkData(iLAS)))
          {
		    bVal = true;
          };
          break;
        case las.lasStandardTools._DTSF:
          if ((!st.checkData(_DTSF)) && (stIN.checkData(iLAS)))
          {
		    bVal = true;
          };
          break;
        case las.lasStandardTools._DTSS:
          if ((!st.checkData(_DTSS)) && (stIN.checkData(iLAS)))
          {
		    bVal = true;
          };
          break;
        case las.lasStandardTools._SPHI:
          if ((!st.checkData(_SPHI)) && (stIN.checkData(iLAS)))
          {
		    bVal = true;
          };
          break;

        case las.lasStandardTools._TH:
          if ((!st.checkData(_TH)) && (stIN.checkData(iLAS)))
          {
		    bVal = true;
          };
          break;
        case las.lasStandardTools._U:
          if ((!st.checkData(_U)) && (stIN.checkData(iLAS)))
          {
		    bVal = true;
          };
          break;
        case las.lasStandardTools._K:
          if ((!st.checkData(_K)) && (stIN.checkData(iLAS)))
          {
		    bVal = true;
          };
          break;
      }
	}

	return (bVal);
  }

  /** Method getZekeID()
   *  <p> This method will turn on the curve identifiers in the Zonation
   *      data structure.
   *  @param  iLAS    = the LAS Column Identifier
   *  @param  stIN    = the LAS File Data Structure
   *  @param  st      = the Zonation File Data Structure
   *  @return iColumn = Column Number in Zonation data structure.
   */

  private static int getZekeID( int iLAS,
	                            lasFileDataStruct stIN,
                                eventFileDataStruct st )
  {
	int iColumn = -1;

    if ((stIN != null) && (st != null))
    {
      switch (iLAS)
      {
        case las.lasStandardTools._GR:
          if (stIN.checkData(iLAS))
          {
	        iColumn = _GR;
          }
          break;
        case las.lasStandardTools._CGR:
          if (stIN.checkData(iLAS))
          {
	  	    iColumn = _CGR;
          }
          break;

        case las.lasStandardTools._PE:
          if (stIN.checkData(iLAS))
          {
		    iColumn = _PE;
          }
          break;
        case las.lasStandardTools._RHOB:
          if (stIN.checkData(iLAS))
          {
		    iColumn = _RHOB;
          }
          break;
        case las.lasStandardTools._DPHI:
          if (stIN.checkData(iLAS))
          {
		    iColumn = _DPHI;
          }
          break;
        case las.lasStandardTools._NPHI:
          if (stIN.checkData(iLAS))
          {
		    iColumn = _NPHI;
          }
          break;

        case las.lasStandardTools._DT:
          if (stIN.checkData(iLAS))
          {
		    iColumn = _DTC;
          }
          break;
        case las.lasStandardTools._DTS:
          if (stIN.checkData(iLAS))
          {
		    iColumn = _DTS;
          }
          break;
        case las.lasStandardTools._DTSF:
          if (stIN.checkData(iLAS))
          {
		    iColumn = _DTSF;
          }
          break;
        case las.lasStandardTools._DTSS:
          if (stIN.checkData(iLAS))
          {
		    iColumn = _DTSS;
          }
          break;
        case las.lasStandardTools._SPHI:
          if (stIN.checkData(iLAS))
          {
		    iColumn = _SPHI;
          }
          break;

        case las.lasStandardTools._TH:
          if (stIN.checkData(iLAS))
          {
		    iColumn = _TH;
          }
          break;
        case las.lasStandardTools._U:
          if (stIN.checkData(iLAS))
          {
		    iColumn = _U;
          }
          break;
        case las.lasStandardTools._K:
          if (stIN.checkData(iLAS))
          {
		    iColumn = _K;
          }
          break;
      }
	}

	return (iColumn);
  }

  /** Method setZekeIdentifiers()
   *  <p> This method will turn on the curve identifiers in the Zonation
   *      data structure.
   */

  private static eventFileDataStruct setZekeIdentifiers( lasFileDataStruct stIN,
                                                          eventFileDataStruct st )
  {
    if ((stIN != null) && (st != null))
    {
      for (int k=0; k<las.lasStandardTools.TOTAL; k++)
      {
        switch (k)
        {
          case las.lasStandardTools._GR:
            if ((!st.checkData(_GR)) && (stIN.checkData(k)))
            {
			  st.iGR = stIN.iGR;
            }
            break;
          case las.lasStandardTools._CGR:
            if ((!st.checkData(_CGR)) && (stIN.checkData(k)))
            {
		      st.iCGR = stIN.iCGR;
            }
            break;

          case las.lasStandardTools._PE:
            if ((!st.checkData(_PE)) && (stIN.checkData(k)))
            {
			  st.iPE = stIN.iPE;
            }
            break;
          case las.lasStandardTools._RHOB:
            if ((!st.checkData(_RHOB)) && (stIN.checkData(k)))
            {
			  st.iRHOB = stIN.iRHOB;
            }
            break;
          case las.lasStandardTools._DPHI:
            if ((!st.checkData(_DPHI)) && (stIN.checkData(k)))
            {
			  st.iDPHI = stIN.iDPHI;
            }
            break;
          case las.lasStandardTools._NPHI:
            if ((!st.checkData(_NPHI)) && (stIN.checkData(k)))
            {
			  st.iNPHI = stIN.iNPHI;
            }
            break;

          case las.lasStandardTools._DT:
            if ((!st.checkData(_DTC)) && (stIN.checkData(k)))
            {
			  st.iDTC = stIN.iDT;
            }
            break;
          case las.lasStandardTools._DTS:
            if ((!st.checkData(_DTS)) && (stIN.checkData(k)))
            {
			  st.iDTS = stIN.iDTS;
            }
            break;
          case las.lasStandardTools._DTSF:
            if ((!st.checkData(_DTSF)) && (stIN.checkData(k)))
            {
			  st.iDTSF = stIN.iDTSF;
            }
            break;
          case las.lasStandardTools._DTSS:
            if ((!st.checkData(_DTSS)) && (stIN.checkData(k)))
            {
			  st.iDTSS = stIN.iDTSS;
            }
            break;
          case las.lasStandardTools._SPHI:
            if ((!st.checkData(_SPHI)) && (stIN.checkData(k)))
            {
			  st.iSPHI = stIN.iSPHI;
            }
            break;

          case las.lasStandardTools._TH:
            if ((!st.checkData(_TH)) && (stIN.checkData(k)))
            {
			  st.iTh = stIN.iTh;
            }
            break;
          case las.lasStandardTools._U:
            if ((!st.checkData(_U)) && (stIN.checkData(k)))
            {
			  st.iU = stIN.iU;
            }
            break;
          case las.lasStandardTools._K:
            if ((!st.checkData(_K)) && (stIN.checkData(k)))
            {
			  st.iK = stIN.iK;
            }
            break;
        }
      }
	}

	return (st);
  }
}

/*
 *  @version 1.1 04/03/2015
 *  @author  John Victorine
 */
