/*
 * @eventXDataUtility.java Version 1.1 05/13/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package event;

import event.eventXDataListStruct;   // Multiple Event List Structure
import event.eventXDataStruct;       // Multiple Event Data Structure

import iqstrat.iqstratHeadersStruct; // Header Information Structure
import las.lasFileDataStruct;        // Log File Data Structure
import event.eventFileDataStruct;    // Event File Data Structure

/** Class eventXDataUtility
 *  <p> This Class will provide basic utility for the data structures
 *
 *  @version 1.1 05/13/2015
 *  @author  John R. Victorine
 */

public class eventXDataUtility
{
  /** Method exists()
   * <p> This method will decide if the data is already present or not
   * @param  stBlock = The Multiple Event Block List Data Structure
   * @param  st      = The Multiple Event List Data Structure
   * @return bValue  = true the data is already available, false it is not.
   */

  public static boolean exists( eventXDataStruct stBlock,
                                eventXDataListStruct st )
  {
	boolean bValue = false;

	if ((st != null) && (stBlock != null))
	{
	  for (int i=0; i<st.iCount; i++)
	  {
	    if ((st.stItem[i].stHeader.sAPI.equals(stBlock.stHeader.sAPI)) &&
	        (stBlock.stHeader.sAPI.length() > 1))
	    {
		  bValue = true;
	    }

	    if ((st.stItem[i].stHeader.sName.toUpperCase().equals(
		        stBlock.stHeader.sName.toUpperCase())) &&
            (stBlock.stHeader.sName.length() > 1))
	    {
		  bValue = true;
	    }
	  }
	}

	return (bValue);
  }

  /** Method setKEY()
   * <p> This method will create a new KEY for the Well Data
   * @param  st   = Multiple Event Data Structure
   * @param  sKEY = Unique KEY for the Well Data.
   * @return st   = Multiple Event Data Structure
   */

  public static eventXDataStruct setKEY( eventXDataStruct st )
  {
	String sKEY = cmn.cmnString.UniqueName();

    st.sKEY          = new String( sKEY );
	st.stHeader.sKEY = new String( sKEY );

	return (st);
  }

  /** Method copyList()
   * <p> This method will copy one structure to another
   * @param  stOld = the Old List Structure
   * @return stNew = the New List structure
   */

  public static eventXDataListStruct copyList( eventXDataListStruct stOld )
  {
    eventXDataListStruct stNew = null;

    if (stOld != null)
    {
      stNew            = new eventXDataListStruct();
      stNew.iCount     = stOld.iCount;
      stNew.stItem     = new eventXDataStruct[stOld.iCount];

      for (int i=0; i<stOld.iCount; i++)
      {
        stNew.stItem[i] = copy( stOld.stItem[i] );
      }
    }

    return (stNew);
  }

  /** Method copy()
   * <p> This method will copy one structure to another
   * @param  stOld = Multiple Event Data Structure to be copied
   * @return stNew = New Multiple Event Data Structure
   */

  public static eventXDataStruct copy( eventXDataStruct stOld )
  {
    int i,j;
    eventXDataStruct stNew = null;

    if (stOld != null)
    {
      stNew            = new eventXDataStruct();

      stNew.sKEY       = new String (stOld.sKEY );

      stNew.depthStart = stOld.depthStart;
      stNew.depthEnd   = stOld.depthEnd;

      // Well Information Data

      stNew.stHeader = iqstrat.iqstratHeadersUtility.copy( stOld.stHeader );

      // LAS File Data

      stNew.stLAS    = las.lasFileDataUtility.copy( stOld.stLAS );

      // Event File Data

      stNew.stZeke   = event.eventFileDataUtility.copy( stOld.stZeke );
    }

    return (stNew);
  }

  /** Method add()
   * <p> This method will add the Multiple Event to an existing list
   * @param  stBlock = The Multiple Event Block List Data Structure
   * @param  st      = The Old Multiple Event List Data Structure
   * @return st      = The new Multiple Event List.
   */

  public static eventXDataListStruct add( eventXDataStruct stBlock,
                                          eventXDataListStruct st )
  {
    int i=0;
    int iRecords = 0;
    int iCount   = 0;
    eventXDataListStruct stTemp = null;

    if (st != null)
      iRecords = st.iCount+1;
    else
      iRecords = 1;

    stTemp        = new eventXDataListStruct();
    stTemp.stItem = new eventXDataStruct[iRecords];

    if (st != null)
    {
      if (st.iCount > 0)
      {
        for (i=0; i<st.iCount; i++)
        {
          if (iCount < iRecords)
          {
            stTemp.stItem[iCount] = copy(st.stItem[i]);
            iCount++;
          }
        }
      }

      st.delete();
    }

    stTemp.stItem[iCount] = new eventXDataStruct();
    stTemp.stItem[iCount] = copy(stBlock);
    iCount++;

    stTemp.iCount = iCount;

    st        = new eventXDataListStruct();
    st.stItem = new eventXDataStruct[iCount];
    st.iCount = iCount;

    for (i=0; i<stTemp.iCount; i++)
    {
      st.stItem[i] = copy(stTemp.stItem[i]);
    }

    stTemp.delete();

    return (st);
  }

  /** Method remove()
   * <p> This method will remove a Multiple Event from an existing list
   * @param  sKEY   = The Primary Key of the data to be removed
   * @param  st     = The Old Multiple Event List Data Structure
   * @return st     = The new Multiple Event List.
   */

  public static eventXDataListStruct remove( String sKEY, eventXDataListStruct st )
  {
    int i=0;
    int iRecords = 0;
    int iCount   = 0;
    int iFound   = -1;
    eventXDataListStruct stTemp = null;

    if (st.iCount == 1)
    {
      st.delete();
      st = null;
    }
    else
    {
      if (st != null)
      {
        iRecords = st.iCount-1;

        stTemp        = new eventXDataListStruct();
        stTemp.stItem = new eventXDataStruct[iRecords];

        for (i=0; i<st.iCount; i++)
        {
          if (sKEY.equals(st.stItem[i].sKEY))
          {
            iFound = i;
          }
        }
      }

      if (st != null)
      {
        if (st.iCount > 0)
        {
          for (i=0; i<st.iCount; i++)
          {
            if (iCount < iRecords)
            {
              if (i != iFound)
              {
                stTemp.stItem[iCount] = copy(st.stItem[i]);
                iCount++;
              }
            }
          }
        }

        st.delete();
      }

      stTemp.iCount = iCount;

      st        = new eventXDataListStruct();
      st.stItem = new eventXDataStruct[iCount];
      st.iCount = iCount;

      for (i=0; i<stTemp.iCount; i++)
      {
        st.stItem[i] = copy(stTemp.stItem[i]);
      }

      stTemp.delete();
    }

    return (st);
  }

  /** Method getElevation( eventXDataStruct st )
   * <p> This method will retrieve the elevation for a specific well/outcrop
   * @param  st    = The cross section data structure
   * @return dElev = The Elevation Depth
   */

  public static double getElevation( eventXDataStruct st )
  {
    double dElev =0.0;

    if (st != null)
    {
      if (st.stHeader != null)
      {
        if (st.stHeader.dGL > 0.0)
          dElev = st.stHeader.dGL;
        else if (st.stHeader.dKB > 0.0)
          dElev = st.stHeader.dKB;
        else
          dElev = st.stHeader.dDF;
      }
    }

    return (dElev);
  }
}


/*
 *  @version 1.1 05/13/2015
 *  @author  John Victorine
 */
