/*
 * @eventMath.java Version 1.1 02/23/2015
 *
 * Copyright (c) 2012 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package event.math;

import java.math.BigDecimal;

/** Class eventMath
 *  <p> This Class provides methods that will compute the sonic data.
 *      This Class supports the Subsurface Seismic Event software.
 *
 *  @version 1.0 02/23/2015
 *  @author  John R. Victorine
 */

public class eventMath
{
  public static final int _NONE       = lith.lithology.lithologySymbolsStruct._NONE;
  public static final int _SAND       = lith.lithology.lithologySymbolsStruct._SAND;
  public static final int _SANDSTONE  = lith.lithology.lithologySymbolsStruct._SANDSTONE;
  public static final int _SHALE      = lith.lithology.lithologySymbolsStruct._SHALE;
  public static final int _SHALY_SAND = lith.lithology.lithologySymbolsStruct._SHALY_SAND;
  // Carbonate Lithotypes
  public static final int _LIMESTONE  = lith.lithology.lithologySymbolsStruct._LIMESTONE;
  public static final int _SHALY_CARB = lith.lithology.lithologySymbolsStruct._SHALY_CARB;
  public static final int _DOLOMITE   = lith.lithology.lithologySymbolsStruct._DOLOMITE;
  public static final int _SHALY_DOLO = lith.lithology.lithologySymbolsStruct._SHALY_DOLO;
  // Evaporites
  public static final int _GYPSUM     = lith.lithology.lithologySymbolsStruct._GYPSUM;
  public static final int _ANHYDRITE  = lith.lithology.lithologySymbolsStruct._ANHYDRITE;
  public static final int _SALT_GENERAL  = lith.lithology.lithologySymbolsStruct._SALT_GENERAL;
  public static final int _HALITE     = lith.lithology.lithologySymbolsStruct._HALITE;
  // Organic-Rich Rocks
  public static final int _COAL       = lith.lithology.lithologySymbolsStruct._COAL;

  public static final int    SST_VC = 0;
  public static final int    SST_C  = 1;
  public static final int    SST_SH = 2;
  public static final int    SHALE  = 3;
  public static final int    LST    = 4;
  public static final int    DOLO   = 5;
  public static final int    SALT   = 6;
  public static final int    ANHY   = 7;
  public static final int    COAL   = 8;

  public static final double VpVs[] = {
	  1.55,  // Very Clean Sandstone
	  1.6,   // Clean Sandstone
	  1.68,  // Shaly Sandstone
	  1.7,   // Compact Shale
	  1.9,   // Limestone
	  1.8,   // Dolomite
	  1.75,  // Salt
	  1.85,  // Anhydrite
	  1.59,  // Coal
   };

  /** Method getAvgDTs()
   *  <p> This method will compute the shear wave travel time (dts) from
   *      the compression wave travel time (dtc) using the DTc/DTs ratio.
   *  @param iRows = the total number of rows
   *  @param dtc   = the compression wave travel time (usec/ft).
   *  @param iLith = the lithology symbol id numbers used to fine the above
   *                 vpvs id numbers from above.
   *  @param dts   = the shear wave trave time (usec/ft).
   */

  public static double[] getAvgDTs(int iRows, double dtss[], double dtsf[])
  {
	double dts[] = null;

    if (iRows > 0)
    {
	  dts = new double[iRows];

	  for (int i=0; i<iRows; i++)
	  {
	    dts[i] = (dtsf[i] + dtss[i]) / 2.0;
	    dts[i] = (double) ( (int) 1000.0 * dts[i] ) / 1000.0;
	  }
	}

	return (dts);
  }

  /** Method getDTs()
   *  <p> This method will compute the shear wave travel time (dts) from
   *      the compression wave travel time (dtc) using the DTc/DTs ratio.
   *  @param iRows = the total number of rows
   *  @param dtc   = the compression wave travel time (usec/ft).
   *  @param iLith = the lithology symbol id numbers used to fine the above
   *                 vpvs id numbers from above.
   *  @param dts   = the shear wave trave time (usec/ft).
   */

  public static double[] getDTs(int iRows, double dtc[], double dLith[])
  {
	double dts[] = null;
	int    id    = -1;

    if (iRows > 0)
    {
	  dts = new double[iRows];

	  for (int i=0; i<iRows; i++)
	  {
        id = -1;
        switch ((int) dLith[i])
        {
          case _NONE:
            break;
          case _SAND:
          case _SANDSTONE:
            id = SST_C;
            break;
          case _SHALE:
            id = SHALE;
            break;
          case _SHALY_SAND:
            id = SST_SH;
            break;
          // Carbonate Lithotypes
          case _LIMESTONE:
            id = LST;
            break;
          case _SHALY_CARB:
            break;
          case _DOLOMITE:
            id = DOLO;
            break;
          case _SHALY_DOLO:
            break;
          // Evaporites
          case _GYPSUM:
            break;
          case _ANHYDRITE:
            id = ANHY;
            break;
          case _SALT_GENERAL:
          case _HALITE:
            id = SALT;
            break;
          // Organic-Rich Rocks
          case _COAL:
            id = COAL;
            break;
  	    }

	    if (id > -1)
	      dts[i] = VpVs[id] * dtc[i];
	    else
	      dts[i] = 1.8 * dtc[i];
	  }
	}

	return (dts);
  }

  /** Method getAvgVelocity()
   *  <p> This method will compute the average velocity for each depth to the
   *      surface.
   */

  public static double[] getAvgVelocity(int iRows, double dTop[], double dBase[],
                                        double dV[], double dNull)
  {
	double dAvg[] = null;
	double depth  = 0.0;

	if (iRows > 0)
	{
      dAvg = new double[iRows];

  	  for (int i=0; i<iRows; i++)
	  {
		dAvg[i] = 0.0;
		depth   = 0.0;

		for (int j=0; j<i; j++)
		{
		  depth   = depth + dBase[j] - dTop[j];
		  dAvg[i] = dAvg[i] + (dBase[j] - dTop[j]) * dV[j];
		}
		dAvg[i] = (double) ( (int) (1000.0 * dAvg[i] / depth) )/1000.0;
	  }
	}

	return (dAvg);
  }

  /** Method getDepthRange()
   *  <p> This method will compute the depth range
   *  @param  iRows   = Number of Rows
   *  @param  dTop    = array of starting depths
   *  @param  dBase   = array of ending depths
   *  @param  dNull   = LAS File Null value
   *  @return dLimits = Minimum & Maximum Depths
   */

  public static double[] getDepthRange(int iRows, double dTop[], double dBase[], double dNull)
  {
	double dLimits[] = { 0.0, 0.0 };

	if (iRows > 0)
	{
  	  for (int i=0; i<iRows; i++)
	  {
		if (i==0)
		{
		  dLimits[0] = dTop[i];
		  dLimits[1] = dBase[i];
		}

		if (dLimits[1] < dTop[i])
		  dLimits[1] = dTop[i];

		if (dLimits[1] < dBase[i])
		  dLimits[1] = dBase[i];
	  }
	}

	return (dLimits);
  }

  /** Method getTime()
   *  <p> This method will compute the array of time values from an array
   *      of depth and sonic transit time.
   *  @param  iRows  = total number of data rows.
   *  @param  depths = array of depth values
   *  @param  dt     = array of Acoustic transit time values
   *  @param  dNull  = the LAS File Null value
   *  @return data   = array of transit time total
   */

  public static double[] getTime( int iRows, double depths[], double dt[], double dNull )
  {
	int i=0;
	double data[] = null;
	double dSum   = 0.0;
//	double dAvg   = 0.0;
    double iFound = 0;

    BigDecimal DTavg  = new BigDecimal("0.0");
    BigDecimal SUM    = new BigDecimal("0.0");
    BigDecimal DTi    = new BigDecimal("0.0");  // BigDecimal Sonic Time
    BigDecimal Ds     = new BigDecimal("0.0");  // BigDecimal Top Depth
    BigDecimal De     = new BigDecimal("0.0");  // BigDecimal End Depth
    BigDecimal DEL    = new BigDecimal("0.0");
    BigDecimal DATA   = new BigDecimal("0.0");
    BigDecimal FACTOR = new BigDecimal("1000.0");

	if (iRows > 1)
	{
	  data = new double[iRows];

	  for (i=0; i<iRows; i++)
	  {
  	    data[i] = 0.0;

		if (dt[i] > dNull)
		{
		  if (iFound == 0)
		  {
//			dAvg  = dt[i];
			DTavg = new BigDecimal(new String(""+dt[i]));
            iFound++;
		  }
		}
	  }

	  for (i=1; i<iRows; i++)
	  {
 	    Ds   = new BigDecimal(new String(""+depths[i-1]));
		De   = new BigDecimal(new String(""+depths[i]));
 	    DEL  = De.subtract(Ds);

		if (dt[i-1] > dNull)
		{
		  DTi  = new BigDecimal(new String(""+dt[i-1]));
		  DATA = DTi.multiply(DEL);

		  // dSum = dSum + dt[i] * (depths[i] - depths[i-1]);
		  SUM  = SUM.add(DTi.multiply(DEL));
	    }
	    else
	    {
		  DATA = DTavg.multiply(DEL);

          // dSum = dSum + dAvg * (depths[i] - depths[i-1]);
		  SUM  = SUM.add(DTavg.multiply(DEL));
		}

		// Round the 1,000.0 * microsec value then
		// Convert from microsec to millisec by dividing by 1,000,000.0

 	    // data[i] = Math.round( 1000.0 * dSum ) / 1000000.0;
        DATA    = SUM.divide(FACTOR, 6, BigDecimal.ROUND_UP);
        data[i] = DATA.doubleValue();
	  }
	}

	return (data);
  }

  /** Method getDeltaT()
   *  <p> This method will compute the array of time difference values
   *      from an array Shear minus Compression time
   *  @param  iRows  = total number of data rows.
   *  @param  dTc    = array of Compression Times
   *  @param  dTs    = array of Shear Times
   *  @param  dNull  = the LAS File Null value
   *  @return data   = Difference between Shear and compression
   */

  public static double[] getDeltaT( int iRows, double dTc[], double dTs[], double dNull )
  {
	int i=0;
	double data[] = null;

    BigDecimal DTC  = new BigDecimal("0.0");
    BigDecimal DTS  = new BigDecimal("0.0");
    BigDecimal DATA = new BigDecimal("0.0");

	if (iRows > 1)
	{
	  data = new double[iRows];

	  for (i=0; i<iRows; i++)
	  {
  	    data[i] = 0.0;
	  }

	  for (i=0; i<iRows; i++)
	  {
		DTC  = new BigDecimal(new String(""+dTc[i]));
		DTS  = new BigDecimal(new String(""+dTs[i]));
		DATA = DTS.subtract(DTC);

		data[i] = DATA.doubleValue();
	  }
	}

	return (data);
  }

  /** Method getVelocity()
   *  <p> This method will compute the array of velocity values for an array
   *      of depth and sonic transit time.
   *  @param  iRows  = total number of data rows.
   *  @param  dt     = array of Acoustic transit time values
   *  @return data   = array of velocity values
   */

  public static double[] getVelocity( int iRows, double dt[], double dNull )
  {
	int i=0;
	double data[] = null;
	double dSum   = 0.0;
    double iFound = 0;

    BigDecimal DTavg  = new BigDecimal("0.0");
    BigDecimal DATA   = new BigDecimal("0.0");
    BigDecimal DTi    = new BigDecimal("0.0"); // BigDecimal Sonic Time
    BigDecimal FACTOR = new BigDecimal("1000000.0");

	if (iRows > 1)
	{
	  data = new double[iRows];

	  for (i=0; i<iRows; i++)
	  {
  	    data[i] = 0.0;

  	    if (dt[i] > dNull)
	    {
	      if (iFound == 0)
	      {
//            DTavg = new BigDecimal(new String(""+dt[i]));
            iFound++;
	      }
  	    }
	  }

	  for (i=0; i<iRows; i++)
	  {
//		if (dt[i] == dNull)
//		  DTi = DTavg;
//		else
		if ((dt[i] > dNull) && (dt[i] > 0.0))
		{
		  DTi = new BigDecimal(new String(""+dt[i]));

		// data[i] = 1000000.0 / dt[i];

		  DATA    = FACTOR.divide(DTi, 6, BigDecimal.ROUND_UP);
		  data[i] = DATA.doubleValue();
		}
	  }
	}

	return (data);
  }

  /** Method getAI()
   *  <p> this method will retrieve an array of Acoustic impedance values
   *  @param  iRows = total number of data rows.
   *  @param  dV     = array of velocity values
   *  @param  dRHOB  = array of bulk density values
   *  @return data   = array of acoustic impedance values
   */

  public static double[] getAI( int iRows, double dV[], double dRHOB[], double dNull )
  {
	int i=0;
	double data[] = null;

    BigDecimal DATA   = new BigDecimal("0.0");
    BigDecimal RHOB   = new BigDecimal("0.0");
    BigDecimal V      = new BigDecimal("0.0");
    BigDecimal V3460  = new BigDecimal("3460.0");
    BigDecimal Vp308  = new BigDecimal("0.308");


	if (iRows > 1)
	{
	  data = new double[iRows];

	  for (i=0; i<iRows; i++)
	  {
  	    data[i] = 0.0;
	  }

	  for (i=0; i<iRows; i++)
	  {
		if (dRHOB[i] != dNull)
		{
		  RHOB = new BigDecimal(new String(""+dRHOB[i]));
		  V    = new BigDecimal(new String(""+dV[i]));

		  // data[i] =  dV[i] * dRHOB[i];

		  DATA    = V.multiply(RHOB);
		  data[i] = DATA.doubleValue();
		}
		else if (dV[i] > 0.0)
		{
		  V    = new BigDecimal(new String(""+dV[i]));

  		  // data[i] = ( dV[i] - 3460.0 ) / 0.308;

  		  DATA    = V.subtract(V3460);
  		  DATA    = DATA.divide(Vp308, 6, BigDecimal.ROUND_UP);
		  data[i] = DATA.doubleValue();
		}
	  }
	}

	return (data);
  }
}

/*
 *  @version 1.1 02/23/2015
 *  @author  John Victorine
 */
